@@ -71,13 +71,14 @@ var LibraryBrowser = {
         Browser.mainLoop.scheduler();
       },
       updateStatus() {
+#if expectToReceiveOnModule('setStatus')
         if (Module['setStatus']) {
           var message = Module['statusMessage'] || 'Please wait...';
           var remaining = Browser.mainLoop.remainingBlockers;
           var expected = Browser.mainLoop.expectedBlockers;
           if (remaining) {
             if (remaining < expected) {
-              Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
+              Module['setStatus'](`{message} ({expected - remaining}/{expected})`);
             } else {
               Module['setStatus'](message);
             }
@@ -85,17 +86,22 @@ var LibraryBrowser = {
             Module['setStatus']('');
           }
         }
+#endif
       },
       runIter(func) {
         if (ABORT) return;
+#if expectToReceiveOnModule('preMainLoop')
         if (Module['preMainLoop']) {
           var preRet = Module['preMainLoop']();
           if (preRet === false) {
             return; // |return false| skips a frame
           }
         }
+#endif
         callUserCallback(func);
+#if expectToReceiveOnModule('postMainLoop')
         Module['postMainLoop']?.();
+#endif
       }
     },
     isFullscreen: false,
@@ -132,7 +138,9 @@ var LibraryBrowser = {
 #endif
         var img = new Image();
         img.onload = () => {
+#if ASSERTIONS
           assert(img.complete, `Image ${name} could not be decoded`);
+#endif
           var canvas = /** @type {!HTMLCanvasElement} */ (document.createElement('canvas'));
           canvas.width = img.width;
           canvas.height = img.height;
@@ -296,8 +304,9 @@ var LibraryBrowser = {
       if (!ctx) return null;
 
       if (setInModule) {
+#if ASSERTIONS
         if (!useWebGL) assert(typeof GLctx == 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
-
+#endif
         Module.ctx = ctx;
         if (useWebGL) GL.makeContextCurrent(contextHandle);
         Module.useWebGL = useWebGL;
@@ -584,16 +593,9 @@ var LibraryBrowser = {
           Browser.mouseMovementY = Browser.getMovementY(event);
         }
 
-        // check if SDL is available
-        if (typeof SDL != "undefined") {
-          Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
-          Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
-        } else {
-          // just add the mouse delta to the current absolut mouse position
-          // FIXME: ideally this should be clamped against the canvas size and zero
-          Browser.mouseX += Browser.mouseMovementX;
-          Browser.mouseY += Browser.mouseMovementY;
-        }
+        // add the mouse delta to the current absolute mouse position
+        Browser.mouseX += Browser.mouseMovementX;
+        Browser.mouseY += Browser.mouseMovementY;
       } else {
         if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
           var touch = event.touch;
@@ -777,8 +779,9 @@ var LibraryBrowser = {
       return onerror ? onerror() : undefined;
     }
 #endif
+#if ASSERTIONS
     assert(runDependencies === 0, 'async_load_script must be run when no other dependencies are active');
-
+#endif
     {{{ runtimeKeepalivePush() }}}
 
     var loadDone = () => {
@@ -905,8 +908,9 @@ var LibraryBrowser = {
    * @param {boolean=} noSetTiming
    */`,
   $setMainLoop: (browserIterationFunc, fps, simulateInfiniteLoop, arg, noSetTiming) => {
+#if ASSERTIONS
     assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');
-
+#endif
     Browser.mainLoop.func = browserIterationFunc;
     Browser.mainLoop.arg = arg;
 
@@ -1285,37 +1289,37 @@ var LibraryBrowser = {
     return info.awaited;
   },
 
-  emscripten_get_preloaded_image_data__deps: ['$PATH_FS', 'malloc'],
+  emscripten_get_preloaded_image_data__deps: ['$getPreloadedImageData', '$UTF8ToString'],
   emscripten_get_preloaded_image_data__proxy: 'sync',
-  emscripten_get_preloaded_image_data: (path, w, h) => {
-    if ((path | 0) === path) path = UTF8ToString(path);
+  emscripten_get_preloaded_image_data: (path, w, h) => getPreloadedImageData(UTF8ToString(path), w, h),
 
+  $getPreloadedImageData__internal: true,
+  $getPreloadedImageData__data: ['$PATH_FS', 'malloc'],
+  $getPreloadedImageData: (path, w, h) => {
     path = PATH_FS.resolve(path);
 
     var canvas = /** @type {HTMLCanvasElement} */(preloadedImages[path]);
-    if (canvas) {
-      var ctx = canvas.getContext("2d");
-      var image = ctx.getImageData(0, 0, canvas.width, canvas.height);
-      var buf = _malloc(canvas.width * canvas.height * 4);
+    if (!canvas) return 0;
 
-      HEAPU8.set(image.data, buf);
+    var ctx = canvas.getContext("2d");
+    var image = ctx.getImageData(0, 0, canvas.width, canvas.height);
+    var buf = _malloc(canvas.width * canvas.height * 4);
 
-      {{{ makeSetValue('w', '0', 'canvas.width', 'i32') }}};
-      {{{ makeSetValue('h', '0', 'canvas.height', 'i32') }}};
-      return buf;
-    }
+    HEAPU8.set(image.data, buf);
 
-    return 0;
+    {{{ makeSetValue('w', '0', 'canvas.width', 'i32') }}};
+    {{{ makeSetValue('h', '0', 'canvas.height', 'i32') }}};
+    return buf;
   },
 
 #if !WASMFS // WasmFS implements this in wasm
-  emscripten_get_preloaded_image_data_from_FILE__deps: ['emscripten_get_preloaded_image_data', 'fileno'],
+  emscripten_get_preloaded_image_data_from_FILE__deps: ['$getPreloadedImageData', 'fileno'],
   emscripten_get_preloaded_image_data_from_FILE__proxy: 'sync',
   emscripten_get_preloaded_image_data_from_FILE: (file, w, h) => {
     var fd = _fileno(file);
     var stream = FS.getStream(fd);
     if (stream) {
-      return _emscripten_get_preloaded_image_data(stream.path, w, h);
+      return getPreloadedImageData(stream.path, w, h);
     }
 
     return 0;
