@@ -33,15 +33,15 @@ var Module =
 var Module = {{{ EXPORT_NAME }}};
 #endif
 
-#if MODULARIZE && EXPORT_READY_PROMISE
+#if MODULARIZE && USE_READY_PROMISE
 // Set up the promise that indicates the Module is initialized
 var readyPromiseResolve, readyPromiseReject;
-Module['ready'] = new Promise((resolve, reject) => {
+var readyPromise = new Promise((resolve, reject) => {
   readyPromiseResolve = resolve;
   readyPromiseReject = reject;
 });
 #if ASSERTIONS
-{{{ addReadyPromiseAssertions("Module['ready']") }}}
+{{{ addReadyPromiseAssertions() }}}
 #endif
 #endif
 
@@ -81,57 +81,37 @@ if (ENVIRONMENT_IS_NODE && ENVIRONMENT_IS_SHELL) {
 }
 #endif
 
-#if !SINGLE_FILE
-#if ENVIRONMENT_MAY_BE_NODE && ((WASM == 1 && (!WASM2JS || !MEM_INIT_IN_WASM)) || WASM == 2)
-// Wasm or Wasm2JS loading:
-
-if (ENVIRONMENT_IS_NODE) {
-  var fs = require('fs');
-#if WASM == 2
-  if (typeof WebAssembly != 'undefined') Module['wasm'] = fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm');
-  else eval(fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm.js')+'');
-#else
-#if !WASM2JS
-  Module['wasm'] = fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm');
-#endif
-#endif
-#if !MEM_INIT_IN_WASM
-  Module['mem'] = fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.mem');
-#endif
-}
-#endif
-
-#if ENVIRONMENT_MAY_BE_SHELL && ((WASM == 1 && (!WASM2JS || !MEM_INIT_IN_WASM)) || WASM == 2)
-if (ENVIRONMENT_IS_SHELL) {
-#if WASM == 2
-  if (typeof WebAssembly != 'undefined') Module['wasm'] = read('{{{ TARGET_BASENAME }}}.wasm', 'binary');
-  else eval(read('{{{ TARGET_BASENAME }}}.wasm.js')+'');
-#else
-#if !WASM2JS
-  Module['wasm'] = read('{{{ TARGET_BASENAME }}}.wasm', 'binary');
-#endif
-#endif
-#if !MEM_INIT_IN_WASM
-  Module['mem'] = read('{{{ TARGET_BASENAME }}}.mem', 'binary');
-#endif
-}
-#endif
-
-#endif // !SINGLE_FILE
-
 // Redefine these in a --pre-js to override behavior. If you would like to
 // remove out() or err() altogether, you can no-op it out to function() {},
 // and build with --closure 1 to get Closure optimize out all the uses
 // altogether.
 
+#if ENVIRONMENT_MAY_BE_NODE && PTHREADS
+// Set up the out() and err() hooks, which are how we can print to stdout or
+// stderr, respectively.
+// Normally just binding console.log/console.error here works fine, but
+// under node (with workers) we see missing/out-of-order messages so route
+// directly to stdout and stderr.
+// See https://github.com/emscripten-core/emscripten/issues/14804
+var defaultPrint = console.log.bind(console);
+var defaultPrintErr = console.error.bind(console);
+if (ENVIRONMENT_IS_NODE) {
+  var fs = require('fs');
+  defaultPrint = (...args) => fs.writeSync(1, args.join(' ') + '\n');
+  defaultPrintErr = (...args) => fs.writeSync(2, args.join(' ') + '\n');
+}
+var out = defaultPrint;
+var err = defaultPrintErr;
+#else
 var out = (text) => console.log(text);
 var err = (text) => console.error(text);
+#endif
 
 // Override this function in a --pre-js file to get a signal for when
 // compilation is ready. In that callback, call the function run() to start
 // the program.
 function ready() {
-#if MODULARIZE && EXPORT_READY_PROMISE
+#if MODULARIZE && USE_READY_PROMISE
   readyPromiseResolve(Module);
 #endif // MODULARIZE
 #if INVOKE_RUN && HAS_MAIN
@@ -154,27 +134,76 @@ function ready() {
 #endif
 #endif
 
+#if PTHREADS
+// MINIMAL_RUNTIME does not support --proxy-to-worker option, so Worker and Pthread environments
+// coincide.
+var ENVIRONMENT_IS_WORKER = typeof importScripts == 'function';
+var ENVIRONMENT_IS_PTHREAD = ENVIRONMENT_IS_WORKER && self.name == 'em-pthread';
+
+#if !MODULARIZE
+// In MODULARIZE mode _scriptName needs to be captured already at the very top of the page immediately when the page is parsed, so it is generated there
+// before the page load. In non-MODULARIZE modes generate it here.
+var _scriptName = (typeof document != 'undefined') ? document.currentScript?.src : undefined;
+#endif
+
+#if ENVIRONMENT_MAY_BE_NODE
+if (ENVIRONMENT_IS_NODE) {
+  var worker_threads = require('worker_threads');
+  global.Worker = worker_threads.Worker;
+  ENVIRONMENT_IS_WORKER = !worker_threads.isMainThread;
+  // Under node we set `workerData` to `em-pthread` to signal that the worker
+  // is hosting a pthread.
+  ENVIRONMENT_IS_PTHREAD = ENVIRONMENT_IS_WORKER && worker_threads['workerData'] == 'em-pthread'
+  _scriptName = __filename;
+} else
+#endif
+if (ENVIRONMENT_IS_WORKER) {
+  _scriptName = self.location.href;
+}
+#endif // PTHREADS
+
 // --pre-jses are emitted after the Module integration code, so that they can
 // refer to Module (if they choose; they can also define Module)
 {{{ preJS() }}}
 
+#if !SINGLE_FILE
+
 #if PTHREADS
-
-#if !MODULARIZE
-// In MODULARIZE mode _scriptDir needs to be captured already at the very top of the page immediately when the page is parsed, so it is generated there
-// before the page load. In non-MODULARIZE modes generate it here.
-var _scriptDir = (typeof document != 'undefined' && document.currentScript) ? document.currentScript.src : undefined;
+if (!ENVIRONMENT_IS_PTHREAD) {
 #endif
 
-// MINIMAL_RUNTIME does not support --proxy-to-worker option, so Worker and Pthread environments
-// coincide.
-#if WASM_WORKERS
-var ENVIRONMENT_IS_WORKER = typeof importScripts == 'function',
-  ENVIRONMENT_IS_PTHREAD = ENVIRONMENT_IS_WORKER && !ENVIRONMENT_IS_WASM_WORKER;
+#if ENVIRONMENT_MAY_BE_NODE && ((WASM == 1 && !WASM2JS) || WASM == 2)
+// Wasm or Wasm2JS loading:
+
+if (ENVIRONMENT_IS_NODE) {
+  var fs = require('fs');
+#if WASM == 2
+  if (typeof WebAssembly != 'undefined') Module['wasm'] = fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm');
+  else eval(fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm.js')+'');
 #else
-var ENVIRONMENT_IS_WORKER = typeof importScripts == 'function',
-  ENVIRONMENT_IS_PTHREAD = ENVIRONMENT_IS_WORKER;
+#if !WASM2JS
+  Module['wasm'] = fs.readFileSync(__dirname + '/{{{ TARGET_BASENAME }}}.wasm');
+#endif
+#endif
+}
 #endif
 
-var currentScriptUrl = typeof _scriptDir != 'undefined' ? _scriptDir : ((typeof document != 'undefined' && document.currentScript) ? document.currentScript.src : undefined);
-#endif // PTHREADS
+#if ENVIRONMENT_MAY_BE_SHELL && ((WASM == 1 && !WASM2JS) || WASM == 2)
+if (ENVIRONMENT_IS_SHELL) {
+#if WASM == 2
+  if (typeof WebAssembly != 'undefined') Module['wasm'] = read('{{{ TARGET_BASENAME }}}.wasm', 'binary');
+  else eval(read('{{{ TARGET_BASENAME }}}.wasm.js')+'');
+#else
+#if !WASM2JS
+  Module['wasm'] = read('{{{ TARGET_BASENAME }}}.wasm', 'binary');
+#endif
+#endif
+}
+#endif
+
+#if PTHREADS
+}
+#endif
+
+#endif // !SINGLE_FILE
+
