@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <memory>
 #include <string>
+#include <optional>
 #include <emscripten/bind.h>
 
 using namespace emscripten;
@@ -12,6 +13,8 @@ class Test {
   int function_three(const std::string&) { return 1; }
   int function_four(bool x) { return 2; }
 
+  long long_fn(unsigned long a) { return 3; }
+
   int const_fn() const { return 0; }
 
   int getX() const { return x; }
@@ -49,9 +52,13 @@ struct ValArr {
   int x, y, z;
 };
 
+EMSCRIPTEN_DECLARE_VAL_TYPE(CallbackType);
+
 struct ValObj {
   Foo foo;
   Bar bar;
+  CallbackType callback;
+  ValObj() : callback(val::undefined()) {}
 };
 
 class ClassWithConstructor {
@@ -61,6 +68,12 @@ class ClassWithConstructor {
   int fn(int x) { return 0; }
 };
 
+class ClassWithTwoConstructors {
+ public:
+  ClassWithTwoConstructors() {}
+  ClassWithTwoConstructors(int) {}
+};
+
 class ClassWithSmartPtrConstructor {
  public:
   ClassWithSmartPtrConstructor(int, const ValArr&) {}
@@ -72,8 +85,6 @@ int smart_ptr_function(std::shared_ptr<ClassWithSmartPtrConstructor>) {
   return 0;
 }
 
-EMSCRIPTEN_DECLARE_VAL_TYPE(CallbackType);
-
 int function_with_callback_param(CallbackType ct) {
   ct(val("hello"));
   return 0;
@@ -81,6 +92,22 @@ int function_with_callback_param(CallbackType ct) {
 
 int global_fn(int, int) { return 0; }
 
+std::string string_test(std::string arg) {
+  return "hi";
+}
+
+std::wstring wstring_test(std::wstring arg) {
+  return L"hi";
+}
+
+std::optional<int> optional_test(std::optional<Foo> arg) {
+  return {};
+}
+
+std::optional<int> optional_and_nonoptional_test(std::optional<Foo> arg1, int arg2) {
+  return {};
+}
+
 class BaseClass {
  public:
   virtual ~BaseClass() = default;
@@ -101,6 +128,7 @@ EMSCRIPTEN_BINDINGS(Test) {
       .function("functionFour", &Test::function_four)
       .function("functionFive(x, y)", &Test::function_one)
       .function("functionSix(str)", &Test::function_three)
+      .function("longFn", &Test::long_fn)
       .function("constFn", &Test::const_fn)
       .property("x", &Test::getX, &Test::setX)
       .property("y", &Test::getY)
@@ -139,18 +167,34 @@ EMSCRIPTEN_BINDINGS(Test) {
 
   value_object<ValObj>("ValObj")
       .field("foo", &ValObj::foo)
-      .field("bar", &ValObj::bar);
+      .field("bar", &ValObj::bar)
+      .field("callback", &ValObj::callback);
 
   register_vector<int>("IntVec");
 
+  register_map<int, int>("MapIntInt");
+
   class_<Foo>("Foo").function("process", &Foo::process);
 
   function("global_fn", &global_fn);
 
+  register_optional<int>();
+  register_optional<Foo>();
+  function("optional_test", &optional_test);
+  function("optional_and_nonoptional_test", &optional_and_nonoptional_test);
+
+  function("string_test", &string_test);
+  function("wstring_test", &wstring_test);
+
   class_<ClassWithConstructor>("ClassWithConstructor")
       .constructor<int, const ValArr&>()
       .function("fn", &ClassWithConstructor::fn);
 
+  // The last defined constructor should be used in the definition.
+  class_<ClassWithTwoConstructors>("ClassWithTwoConstructors")
+      .constructor<>()
+      .constructor<int>();
+
   class_<ClassWithSmartPtrConstructor>("ClassWithSmartPtrConstructor")
       .smart_ptr_constructor(
           "ClassWithSmartPtrConstructor",
