@@ -98,6 +98,7 @@ misc_test_modes = [
   'bigint',
   'browser64',
   'browser64_4gb',
+  'browser_2gb',
 ]
 
 
@@ -134,7 +135,7 @@ def get_all_tests(modules):
 
 
 def get_crossplatform_tests(modules):
-  suites = ['core2', 'other'] # We don't need all versions of every test
+  suites = ['core0', 'other'] # We don't need all versions of every test
   crossplatform_tests = []
   # Walk over the test suites and find the test functions with the
   # is_crossplatform_test attribute applied by @crossplatform decorator
@@ -266,7 +267,7 @@ def error_on_legacy_suite_names(args):
       utils.exit_with_error('`%s` test suite has been replaced with `%s`', a, new)
 
 
-def load_test_suites(args, modules, start_at):
+def load_test_suites(args, modules, start_at, repeat):
   found_start = not start_at
 
   loader = unittest.TestLoader()
@@ -295,8 +296,9 @@ def load_test_suites(args, modules, start_at):
             found_start = True
           else:
             continue
-        total_tests += 1
-        suite.addTest(test)
+        for _x in range(repeat):
+          total_tests += 1
+          suite.addTest(test)
       suites.append((m.__name__, suite))
   if not found_start:
     utils.exit_with_error(f'unable to find --start-at test: {start_at}')
@@ -364,30 +366,32 @@ def run_tests(options, suites):
 
 def parse_args(args):
   parser = argparse.ArgumentParser(prog='runner.py', description=__doc__)
-  parser.add_argument('--save-dir', action='store_true', default=None,
+  parser.add_argument('--save-dir', action='store_true',
                       help='Save the temporary directory used during for each '
                            'test.  Implies --cores=1.  Defaults to true when running a single test')
   parser.add_argument('--no-clean', action='store_true',
                       help='Do not clean the temporary directory before each test run')
-  parser.add_argument('--verbose', '-v', action='store_true', default=None)
-  parser.add_argument('--all-engines', action='store_true', default=None)
-  parser.add_argument('--detect-leaks', action='store_true', default=None)
+  parser.add_argument('--verbose', '-v', action='store_true')
+  parser.add_argument('--all-engines', action='store_true')
+  parser.add_argument('--detect-leaks', action='store_true')
   parser.add_argument('--skip-slow', action='store_true', help='Skip tests marked as slow')
   parser.add_argument('--cores', '-j',
                       help='Set the number tests to run in parallel.  Defaults '
                            'to the number of CPU cores.', default=None)
-  parser.add_argument('--rebaseline', action='store_true', default=None,
+  parser.add_argument('--rebaseline', action='store_true',
                       help='Automatically update test expectations for tests that support it.')
   parser.add_argument('--browser',
                       help='Command to launch web browser in which to run browser tests.')
   parser.add_argument('tests', nargs='*')
-  parser.add_argument('--failfast', action='store_const', const=True, default=False)
+  parser.add_argument('--failfast', action='store_true')
   parser.add_argument('--start-at', metavar='NAME', help='Skip all tests up until <NAME>')
   parser.add_argument('--continue', dest='_continue', action='store_true',
                       help='Resume from the last run test.'
                            'Useful when combined with --failfast')
-  parser.add_argument('--force64', action='store_const', const=True, default=None)
+  parser.add_argument('--force64', action='store_true')
   parser.add_argument('--crossplatform-only', action='store_true')
+  parser.add_argument('--repeat', type=int, default=1,
+                      help='Repeat each test N times (default: 1).')
   return parser.parse_args()
 
 
@@ -465,6 +469,7 @@ def main(args):
   all_tests = get_all_tests(modules)
   if options.crossplatform_only:
     tests = get_crossplatform_tests(modules)
+    skip_requested_tests(options.tests, modules)
   else:
     tests = tests_with_expanded_wildcards(tests, all_tests)
     tests = skip_requested_tests(tests, modules)
@@ -474,7 +479,7 @@ def main(args):
     if os.path.exists(common.LAST_TEST):
       options.start_at = utils.read_file(common.LAST_TEST).strip()
 
-  suites, unmatched_tests = load_test_suites(tests, modules, options.start_at)
+  suites, unmatched_tests = load_test_suites(tests, modules, options.start_at, options.repeat)
   if unmatched_tests:
     print('ERROR: could not find the following tests: ' + ' '.join(unmatched_tests))
     return 1
