@@ -10,6 +10,7 @@ sys.path.insert(0, __rootdir__)
 from . import shared
 from . import line_endings
 from . import utils
+from . import feature_matrix
 from .settings import settings
 
 logger = logging.getLogger('minimal_runtime_shell')
@@ -23,7 +24,7 @@ def generate_minimal_runtime_load_statement(target_basename):
   # Depending on whether streaming Wasm compilation is enabled or not, the minimal sized code to download Wasm looks a bit different.
   # Expand {{{ DOWNLOAD_WASM }}} block from here (if we added #define support, this could be done in the template directly)
   if settings.MINIMAL_RUNTIME_STREAMING_WASM_COMPILATION:
-    if settings.MIN_SAFARI_VERSION != settings.TARGET_NOT_SUPPORTED or settings.ENVIRONMENT_MAY_BE_NODE or settings.MIN_FIREFOX_VERSION < 58 or settings.MIN_CHROME_VERSION < 61:
+    if settings.MIN_SAFARI_VERSION != feature_matrix.UNSUPPORTED or settings.ENVIRONMENT_MAY_BE_NODE or settings.MIN_FIREFOX_VERSION < 58 or settings.MIN_CHROME_VERSION < 61:
       # Firefox 52 added Wasm support, but only Firefox 58 added compileStreaming.
       # Chrome 57 added Wasm support, but only Chrome 61 added compileStreaming.
       # https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/WebAssembly/compileStreaming
@@ -34,7 +35,7 @@ def generate_minimal_runtime_load_statement(target_basename):
       download_wasm = "WebAssembly.compileStreaming(fetch('%s'))" % (target_basename + '.wasm')
   elif settings.MINIMAL_RUNTIME_STREAMING_WASM_INSTANTIATION:
     # Same compatibility story as above for https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/WebAssembly/instantiateStreaming
-    if settings.MIN_SAFARI_VERSION != settings.TARGET_NOT_SUPPORTED or settings.ENVIRONMENT_MAY_BE_NODE or settings.MIN_FIREFOX_VERSION < 58 or settings.MIN_CHROME_VERSION < 61:
+    if settings.MIN_SAFARI_VERSION != feature_matrix.UNSUPPORTED or settings.ENVIRONMENT_MAY_BE_NODE or settings.MIN_FIREFOX_VERSION < 58 or settings.MIN_CHROME_VERSION < 61:
       download_wasm = "!WebAssembly.instantiateStreaming && binary('%s')" % (target_basename + '.wasm')
     else:
       # WebAssembly.instantiateStreaming() is unconditionally supported, so we do not download wasm in the .html file,
@@ -45,14 +46,6 @@ def generate_minimal_runtime_load_statement(target_basename):
 
   files_to_load = ["script('%s')" % (target_basename + '.js')] # Main JS file always in first entry
 
-  # Download separate memory initializer file .mem
-  if not settings.MEM_INIT_IN_WASM:
-    if settings.MODULARIZE:
-      modularize_imports += ['mem: r[%d]' % len(files_to_load)]
-    else:
-      then_statements += ["%s.mem = r[%d];" % (settings.EXPORT_NAME, len(files_to_load))]
-    files_to_load += ["binary('%s')" % (target_basename + '.mem')]
-
   # Download .wasm file
   if (settings.WASM == 1 and settings.WASM2JS == 0) or not download_wasm:
     if settings.MODULARIZE:
@@ -75,9 +68,6 @@ def generate_minimal_runtime_load_statement(target_basename):
     if download_wasm and settings.WASM_WORKERS == 1:
       files_to_load += ["binary('%s')" % (target_basename + '.ww.js')]
 
-  if settings.MODULARIZE and settings.PTHREADS:
-    modularize_imports += ["worker: '{{{ PTHREAD_WORKER_FILE }}}'"]
-
   # Download Wasm2JS code if target browser does not support WebAssembly
   if settings.WASM == 2:
     if settings.MODULARIZE:
@@ -201,7 +191,7 @@ def generate_minimal_runtime_html(target, options, js_target, target_basename):
 
   shell = shell.replace('{{{ TARGET_BASENAME }}}', target_basename)
   shell = shell.replace('{{{ EXPORT_NAME }}}', settings.EXPORT_NAME)
-  shell = shell.replace('{{{ PTHREAD_WORKER_FILE }}}', settings.PTHREAD_WORKER_FILE)
+  shell = shell.replace('{{{ TARGET_JS_NAME }}}', settings.TARGET_JS_NAME)
 
   # In SINGLE_FILE build, embed the main .js file into the .html output
   if settings.SINGLE_FILE:
