@@ -80,12 +80,8 @@ def clear(ports, settings, shared):
   shared.cache.erase_lib('libmpg123.a')
 
 
-def process_args(ports):
-  return []
-
-
 def show():
-  return 'mpg123 (USE_MPG123=1; zlib license)'
+  return 'mpg123 (-sUSE_MPG123=1 or --use-port=mpg123; zlib license)'
 
 
 config_h = r'''/* src/config.h.  Generated from config.h.in by configure.  */
