@@ -29,6 +29,8 @@ LIBC_SOCKETS = ['socket.c', 'socketpair.c', 'shutdown.c', 'bind.c', 'connect.c',
                 'listen.c', 'accept.c', 'getsockname.c', 'getpeername.c', 'send.c',
                 'recv.c', 'sendto.c', 'recvfrom.c', 'sendmsg.c', 'recvmsg.c',
                 'getsockopt.c', 'setsockopt.c', 'freeaddrinfo.c',
+                'gethostbyaddr.c', 'gethostbyaddr_r.c', 'gethostbyname.c',
+                'gethostbyname_r.c', 'gethostbyname2.c', 'gethostbyname2_r.c',
                 'in6addr_any.c', 'in6addr_loopback.c', 'accept4.c']
 
 # Experimental: Setting EMCC_USE_NINJA will cause system libraries to get built with ninja rather
@@ -226,7 +228,7 @@ rule archive
     objects = []
     for src in input_files:
       # Resolve duplicates by appending unique.
-      # This is needed on case insensitve filesystem to handle,
+      # This is needed on case insensitive filesystem to handle,
       # for example, _exit.o and _Exit.o.
       object_basename = shared.unsuffixed_basename(src)
       if case_insensitive:
@@ -487,15 +489,17 @@ class Library:
     By default, this builds all the source files returned by `self.get_files()`,
     with the `cflags` returned by `self.get_cflags()`.
     """
+    batch_inputs = int(os.environ.get('EMCC_BATCH_BUILD', '1'))
     batches = {}
     commands = []
     objects = set()
     cflags = self.get_cflags()
     if self.deterministic_paths:
       source_dir = utils.path_from_root()
-      relative_source_dir = os.path.relpath(source_dir, build_dir)
+      if batch_inputs:
+        relative_source_dir = os.path.relpath(source_dir, build_dir)
+        cflags += [f'-ffile-prefix-map={relative_source_dir}/=']
       cflags += [f'-ffile-prefix-map={source_dir}=/emsdk/emscripten',
-                 f'-ffile-prefix-map={relative_source_dir}/=',
                  '-fdebug-compilation-dir=/emsdk/emscripten']
     case_insensitive = is_case_insensitive(build_dir)
     for src in self.get_files():
@@ -531,24 +535,27 @@ class Library:
           object_uuid += 1
           o = os.path.join(build_dir, f'{object_basename}__{object_uuid}.o')
         commands.append(cmd + [src, '-o', o])
-      else:
+      elif batch_inputs:
         # Use relative paths to reduce the length of the command line.
         # This allows to avoid switching to a response file as often.
         src = os.path.relpath(src, build_dir)
         src = utils.normalize_path(src)
         batches.setdefault(tuple(cmd), []).append(src)
+      else:
+        commands.append(cmd + [src, '-o', o])
       objects.add(o)
 
-    # Choose a chunk size that is large enough to avoid too many subprocesses
-    # but not too large to avoid task starvation.
-    # For now the heuristic is to split inputs by 2x number of cores.
-    chunk_size = max(1, len(objects) // (2 * shared.get_num_cores()))
-    # Convert batches to commands.
-    for cmd, srcs in batches.items():
-      cmd = list(cmd)
-      for i in range(0, len(srcs), chunk_size):
-        chunk_srcs = srcs[i:i + chunk_size]
-        commands.append(building.get_command_with_possible_response_file(cmd + chunk_srcs))
+    if batch_inputs:
+      # Choose a chunk size that is large enough to avoid too many subprocesses
+      # but not too large to avoid task starvation.
+      # For now the heuristic is to split inputs by 2x number of cores.
+      chunk_size = max(1, len(objects) // (2 * shared.get_num_cores()))
+      # Convert batches to commands.
+      for cmd, srcs in batches.items():
+        cmd = list(cmd)
+        for i in range(0, len(srcs), chunk_size):
+          chunk_srcs = srcs[i:i + chunk_size]
+          commands.append(building.get_command_with_possible_response_file(cmd + chunk_srcs))
 
     run_build_commands(commands, num_inputs=len(objects), build_dir=build_dir)
     return objects
@@ -568,7 +575,7 @@ class Library:
       if not generate_only:
         run_ninja(build_dir)
     else:
-      # Use a seperate build directory to the ninja flavor so that building without
+      # Use a separate build directory to the ninja flavor so that building without
       # EMCC_USE_NINJA doesn't clobber the ninja build tree
       build_dir += '-tmp'
       utils.safe_ensure_dirs(build_dir)
@@ -871,6 +878,7 @@ class MuslInternalLibrary(Library):
   includes = [
     'system/lib/libc/musl/src/internal',
     'system/lib/libc/musl/src/include',
+    'system/lib/libc/musl/include',
     'system/lib/libc',
     'system/lib/pthread',
   ]
@@ -915,11 +923,36 @@ class libcompiler_rt(MTLibrary, SjLjLibrary):
   # restriction soon: https://reviews.llvm.org/D71738
   force_object_files = True
 
-  cflags = ['-fno-builtin']
+  cflags = ['-fno-builtin', '-DNDEBUG']
   src_dir = 'system/lib/compiler-rt/lib/builtins'
   includes = ['system/lib/libc']
-  # gcc_personality_v0.c depends on libunwind, which don't include by default.
-  src_files = glob_in_path(src_dir, '*.c', excludes=['gcc_personality_v0.c', 'truncdfbf2.c', 'truncsfbf2.c', 'crtbegin.c', 'crtend.c'])
+  excludes = [
+    # gcc_personality_v0.c depends on libunwind, which don't include by default.
+    'gcc_personality_v0.c',
+    # bfloat16
+    'extendbfsf2.c',
+    'truncdfbf2.c',
+    'truncsfbf2.c',
+    # We provide our own crt
+    'crtbegin.c',
+    'crtend.c',
+    # 80-bit long double (xf_float)
+    'divxc3.c',
+    'extendxftf2.c',
+    'fixxfdi.c',
+    'fixxfti.c',
+    'fixunsxfdi.c',
+    'fixunsxfsi.c',
+    'fixunsxfti.c',
+    'floatdixf.c',
+    'floattixf.c',
+    'floatundixf.c',
+    'floatuntixf.c',
+    'mulxc3.c',
+    'powixf2.c',
+    'trunctfxf2.c',
+  ]
+  src_files = glob_in_path(src_dir, '*.c', excludes=excludes)
   src_files += files_in_path(
       path='system/lib/compiler-rt',
       filenames=[
@@ -1058,12 +1091,13 @@ class libc(MuslInternalLibrary,
     ignore += [
         'memcpy.c', 'memset.c', 'memmove.c', 'getaddrinfo.c', 'getnameinfo.c',
         'res_query.c', 'res_querydomain.c',
-        'proto.c', 'gethostbyaddr.c', 'gethostbyaddr_r.c', 'gethostbyname.c',
-        'gethostbyname2_r.c', 'gethostbyname_r.c', 'gethostbyname2.c',
+        'proto.c',
+        'ppoll.c',
         'syscall.c', 'popen.c', 'pclose.c',
         'getgrouplist.c', 'initgroups.c', 'wordexp.c', 'timer_create.c',
         'getentropy.c',
         'getauxval.c',
+        'lookup_name.c',
         # 'process' exclusion
         'fork.c', 'vfork.c', 'posix_spawn.c', 'posix_spawnp.c', 'execve.c', 'waitid.c', 'system.c',
         '_Fork.c',
@@ -1165,7 +1199,7 @@ class libc(MuslInternalLibrary,
 
     ignore = set(ignore)
     for dirpath, dirnames, filenames in os.walk(musl_srcdir):
-      # Don't recurse into ingored directories
+      # Don't recurse into ignored directories
       remove = [d for d in dirnames if d in ignore]
       for r in remove:
         dirnames.remove(r)
@@ -1199,7 +1233,7 @@ class libc(MuslInternalLibrary,
 
     libc_files += files_in_path(
         path='system/lib/libc/musl/src/linux',
-        filenames=['getdents.c', 'gettid.c', 'utimes.c'])
+        filenames=['getdents.c', 'gettid.c', 'utimes.c', 'statx.c'])
 
     libc_files += files_in_path(
         path='system/lib/libc/musl/src/sched',
@@ -1207,7 +1241,7 @@ class libc(MuslInternalLibrary,
 
     libc_files += files_in_path(
         path='system/lib/libc/musl/src/exit',
-        filenames=['_Exit.c', 'atexit.c', 'at_quick_exit.c', 'quick_exit.c'])
+        filenames=['abort.c', '_Exit.c', 'atexit.c', 'at_quick_exit.c', 'quick_exit.c'])
 
     libc_files += files_in_path(
         path='system/lib/libc/musl/src/ldso',
@@ -1252,12 +1286,14 @@ class libc(MuslInternalLibrary,
           'mktime.c',
           'tzset.c',
           'kill.c',
+          'lookup_name.c',
           'pthread_sigmask.c',
           'raise.c',
           'sigaction.c',
           'sigtimedwait.c',
           'wasi-helpers.c',
           'sbrk.c',
+          'system.c',
         ])
 
     if settings.RELOCATABLE:
@@ -1383,7 +1419,7 @@ class libwasm_workers(MTLibrary):
     cflags = super().get_cflags()
     if self.debug:
       cflags += ['-D_DEBUG']
-      # library_wasm_worker.c contains an assert that a nonnull paramater
+      # library_wasm_worker.c contains an assert that a nonnull parameter
       # is no NULL, which llvm now warns is redundant/tautological.
       cflags += ['-Wno-tautological-pointer-compare']
       # Override the `-O2` default.  Building library_wasm_worker.c with
@@ -1534,12 +1570,12 @@ class libcxxabi(NoExceptLibrary, MTLibrary, DebugLibrary):
     if self.eh_mode == Exceptions.NONE:
       cflags.append('-D_LIBCXXABI_NO_EXCEPTIONS')
     elif self.eh_mode == Exceptions.EMSCRIPTEN:
-      cflags.append('-D__USING_EMSCRIPTEN_EXCEPTIONS__')
+      cflags.append('-D__EMSCRIPTEN_EXCEPTIONS__')
       # The code used to interpret exceptions during terminate
       # is not compatible with emscripten exceptions.
       cflags.append('-DLIBCXXABI_SILENT_TERMINATE')
     elif self.eh_mode == Exceptions.WASM:
-      cflags.append('-D__USING_WASM_EXCEPTIONS__')
+      cflags.append('-D__WASM_EXCEPTIONS__')
     return cflags
 
   def get_files(self):
@@ -1607,8 +1643,18 @@ class libcxx(NoExceptLibrary, MTLibrary):
     'support.cpp',
     'int128_builtins.cpp',
     'libdispatch.cpp',
+    # Emscripten does not have C++20's time zone support which requires access
+    # to IANA Time Zone Database. TODO Implement this using JS timezone
+    'tz.cpp',
+    'tzdb_list.cpp',
   ]
 
+  def get_cflags(self):
+    cflags = super().get_cflags()
+    if self.eh_mode == Exceptions.WASM:
+      cflags.append('-D__WASM_EXCEPTIONS__')
+    return cflags
+
 
 class libunwind(NoExceptLibrary, MTLibrary):
   name = 'libunwind'
@@ -1638,9 +1684,9 @@ class libunwind(NoExceptLibrary, MTLibrary):
     if self.eh_mode == Exceptions.NONE:
       cflags.append('-D_LIBUNWIND_HAS_NO_EXCEPTIONS')
     elif self.eh_mode == Exceptions.EMSCRIPTEN:
-      cflags.append('-D__USING_EMSCRIPTEN_EXCEPTIONS__')
+      cflags.append('-D__EMSCRIPTEN_EXCEPTIONS__')
     elif self.eh_mode == Exceptions.WASM:
-      cflags.append('-D__USING_WASM_EXCEPTIONS__')
+      cflags.append('-D__WASM_EXCEPTIONS__')
     return cflags
 
 
@@ -1657,7 +1703,6 @@ class libmalloc(MTLibrary):
     if self.malloc not in ('dlmalloc', 'emmalloc', 'emmalloc-debug', 'emmalloc-memvalidate', 'emmalloc-verbose', 'emmalloc-memvalidate-verbose', 'mimalloc', 'none'):
       raise Exception('malloc must be one of "emmalloc[-debug|-memvalidate][-verbose]", "dlmalloc" or "none", see settings.js')
 
-    self.use_errno = kwargs.pop('use_errno')
     self.is_tracing = kwargs.pop('is_tracing')
     self.memvalidate = kwargs.pop('memvalidate')
     self.verbose = kwargs.pop('verbose')
@@ -1682,8 +1727,6 @@ class libmalloc(MTLibrary):
       cflags += ['-UNDEBUG', '-DDLMALLOC_DEBUG']
     else:
       cflags += ['-DNDEBUG']
-    if not self.use_errno:
-      cflags += ['-DMALLOC_FAILURE_ACTION=', '-DEMSCRIPTEN_NO_ERRNO']
     if self.is_tracing:
       cflags += ['--tracing']
     return cflags
@@ -1695,9 +1738,6 @@ class libmalloc(MTLibrary):
     name = super().get_base_name()
     if self.is_debug and not self.memvalidate and not self.verbose:
       name += '-debug'
-    if not self.use_errno:
-      # emmalloc doesn't actually use errno, but it's easier to build it again
-      name += '-noerrno'
     if self.is_tracing:
       name += '-tracing'
     return name
@@ -1707,14 +1747,13 @@ class libmalloc(MTLibrary):
 
   @classmethod
   def vary_on(cls):
-    return super().vary_on() + ['is_debug', 'use_errno', 'is_tracing', 'memvalidate', 'verbose']
+    return super().vary_on() + ['is_debug', 'is_tracing', 'memvalidate', 'verbose']
 
   @classmethod
   def get_default_variation(cls, **kwargs):
     return super().get_default_variation(
       malloc=settings.MALLOC,
       is_debug=settings.ASSERTIONS >= 2,
-      use_errno=settings.SUPPORT_ERRNO,
       is_tracing=settings.EMSCRIPTEN_TRACING,
       memvalidate='memvalidate' in settings.MALLOC,
       verbose='verbose' in settings.MALLOC,
@@ -1840,6 +1879,13 @@ class libGL(MTLibrary):
     )
 
 
+class libwebgpu(MTLibrary):
+  name = 'libwebgpu'
+
+  src_dir = 'system/lib/webgpu'
+  src_files = ['webgpu.cpp']
+
+
 class libwebgpu_cpp(MTLibrary):
   name = 'libwebgpu_cpp'
 
@@ -2017,7 +2063,14 @@ class libsanitizer_common_rt(CompilerRTLibrary, MTLibrary):
               'system/lib/compiler-rt/lib',
               'system/lib/libc']
   never_force = True
-  cflags = ['-D_LARGEFILE64_SOURCE']
+  cflags = [
+    '-D_LARGEFILE64_SOURCE',
+    # The upstream code has many format violations and suppresses it with
+    # -Wno-format, so we match that.
+    # https://github.com/llvm/llvm-project/blob/da675b922cca3dc9a76642d792e882979a3d8c82/compiler-rt/lib/sanitizer_common/CMakeLists.txt#L225-L226
+    # TODO Remove this when the issues are resolved.
+    '-Wno-format',
+  ]
 
   src_dir = 'system/lib/compiler-rt/lib/sanitizer_common'
   src_glob = '*.cpp'
@@ -2180,7 +2233,7 @@ class libstubs(DebugLibrary):
   src_files = ['emscripten_syscall_stubs.c', 'emscripten_libc_stubs.c']
 
 
-def get_libs_to_link(args, forced, only_forced):
+def get_libs_to_link(args):
   libs_to_link = []
 
   if '-nostdlib' in args:
@@ -2196,11 +2249,19 @@ def get_libs_to_link(args, forced, only_forced):
   # ones you want
   force_include = []
   force = os.environ.get('EMCC_FORCE_STDLIBS')
+  # Setting this will only use the forced libs in EMCC_FORCE_STDLIBS. This
+  # avoids spending time checking for unresolved symbols in your project files,
+  # which can speed up linking, but if you do not have the proper list of
+  # actually needed libraries, errors can occur.
+  only_forced = os.environ.get('EMCC_ONLY_FORCED_STDLIBS')
+  if only_forced:
+    # One of the purposes EMCC_ONLY_FORCED_STDLIBS was to skip the scanning
+    # of the input files for reverse dependencies.
+    diagnostics.warning('deprecated', 'EMCC_ONLY_FORCED_STDLIBS is deprecated.  Use `-nostdlib` to avoid linking standard libraries')
   if force == '1':
     force_include = [name for name, lib in system_libs_map.items() if not lib.never_force]
   elif force is not None:
     force_include = force.split(',')
-  force_include += forced
   if force_include:
     logger.debug(f'forcing stdlibs: {force_include}')
 
@@ -2311,7 +2372,9 @@ def get_libs_to_link(args, forced, only_forced):
     add_library('libsockets')
 
   if settings.USE_WEBGPU:
-    add_library('libwebgpu_cpp')
+    add_library('libwebgpu')
+    if settings.LINK_AS_CXX:
+      add_library('libwebgpu_cpp')
 
   if settings.WASM_WORKERS:
     add_library('libwasm_workers')
@@ -2330,17 +2393,9 @@ def get_libs_to_link(args, forced, only_forced):
   return libs_to_link
 
 
-def calculate(args, forced):
-  # Setting this will only use the forced libs in EMCC_FORCE_STDLIBS. This avoids spending time checking
-  # for unresolved symbols in your project files, which can speed up linking, but if you do not have
-  # the proper list of actually needed libraries, errors can occur.
-  only_forced = os.environ.get('EMCC_ONLY_FORCED_STDLIBS')
-  if only_forced:
-    # One of the purposes EMCC_ONLY_FORCED_STDLIBS was to skip the scanning
-    # of the input files for reverse dependencies.
-    diagnostics.warning('deprecated', 'EMCC_ONLY_FORCED_STDLIBS is deprecated.  Use `-nostdlib` to avoid linking standard libraries')
+def calculate(args):
 
-  libs_to_link = get_libs_to_link(args, forced, only_forced)
+  libs_to_link = get_libs_to_link(args)
 
   # When LINKABLE is set the entire link command line is wrapped in --whole-archive by
   # building.link_ldd.  And since --whole-archive/--no-whole-archive processing does not nest we
@@ -2387,11 +2442,12 @@ def install_system_headers(stamp):
     # Copy the generic arch files first then
     ('lib', 'libc', 'musl', 'arch', 'generic'): '',
     # Then overlay the emscripten directory on top.
-    # This mimicks how musl itself installs its headers.
+    # This mimics how musl itself installs its headers.
     ('lib', 'libc', 'musl', 'arch', 'emscripten'): '',
     ('lib', 'libc', 'musl', 'include'): '',
     ('lib', 'libcxx', 'include'): os.path.join('c++', 'v1'),
     ('lib', 'libcxxabi', 'include'): os.path.join('c++', 'v1'),
+    ('lib', 'mimalloc', 'include'): '',
   }
 
   target_include_dir = cache.get_include_dir()
